<'
# Copyright (c) 2012 LAAS/CNRS
# All rights reserved.
#
# Redistribution and use  in source  and binary  forms,  with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   1. Redistributions of  source  code must retain the  above copyright
#      notice and this list of conditions.
#   2. Redistributions in binary form must reproduce the above copyright
#      notice and  this list of  conditions in the  documentation and/or
#      other materials provided with the distribution.
#
#                                      Anthony Mallet on Thu Aug  2 2012
#

if {[llength $argv] != 1} { error "expected arguments: component" }
lassign $argv component

# compute handy shortcuts
set comp [$component name]
set COMP [string toupper [$component name]]

lang c
'>
#include "autoconf/acheader.h"

#include <cerrno>
#include <cstdlib>
#include <cstdio>
#include <cstring>

#include <vector>

#include "ros/master.h"

#include "<"$comp">/c/client.h"
#include "<"$comp">_msglib.h"
#include "private.h"


/* --- local data ---------------------------------------------------------- */

static char ebuffer[256];


/* <"[--- ${comp}_init --------------------------------------------------]"> */

genom_client
genom_<"$comp">_client_init(int argc, char *argv[])
{
  static char progname[] = "<"$comp">";
  std::vector<std::string> nodes;
  struct ros_options opts;
  char *largv[argc+1];
  genom_state state;
  genom_client h;
  int i, e;

  h = (genom_client)malloc(sizeof(*h));
  if (!h) return NULL;
  h->instance = NULL;
  h->node = NULL;
  h->connections = NULL;
  h->active_rqst = NULL;
  h->nactive = h->next = 0;

  /* must shift args so that getopt() works (argv[0] must be progname) */
  largv[0] = progname;
  for(i=0; i<argc; i++) {
    largv[i+1] = argv[i];
  }

  h->error = ros_getopt(argc+1, largv, &opts, ebuffer, sizeof(ebuffer));
  if (h->error) {
    errno = h->error;
    free(h);
    return NULL;
  }

  h->error = ros_init(argc+1, largv, ebuffer, sizeof(ebuffer));
  if (h->error) goto error;

  h->instance = strdup(opts.instance?opts.instance:"<"$comp">");
  try {
    /* lookup server */
    if (!ros::master::getNodes(nodes)) {
      snprintf(ebuffer, sizeof(ebuffer), "cannot list running nodes");
      h->error = GENOM_CLIENT_ERR_FLAG;
      goto error;
    }
    for(i=e=0; (unsigned)i<nodes.size(); i++) {
      if (!strcmp(h->instance, nodes[i].c_str()+1) && nodes[i][0] == '/') {
        e = 1;
        break;
      }
    }
    if (!e) {
      snprintf(ebuffer, sizeof(ebuffer),
               "%s node does not seem to be running", h->instance);
      h->error = GENOM_CLIENT_ERR_FLAG;
      goto error;
    }

    h->node = new ros::NodeHandle;
  }
  catch (ros::Exception e) {
    snprintf(ebuffer, sizeof(ebuffer), "%s", e.what());
    h->error = GENOM_CLIENT_ERR_FLAG;
    goto error;
  }

  /* check digest */
  e = genom_<"$comp">_client_genom_state_port(h, &state);
  if (e) {
    strerror_r(e, ebuffer, sizeof(ebuffer));
    strncat(ebuffer, " while reading state port",
           sizeof(ebuffer) - 1 - strlen(ebuffer));
    h->error = GENOM_CLIENT_ERR_FLAG; goto error;
  }
  if (strcmp(state.digest, "<"[$component digest]">")) {
    snprintf(ebuffer, sizeof(ebuffer),
             "inconsistent interface digest: server %s (%s), "
             "client <"[$component version]"> (<"[clock format [clock sec]]">)",
             state.version, state.date);
    h->error = GENOM_CLIENT_ERR_FLAG; goto error;
  }

  /* connect to services */
  h->connections = new struct connections;
<'foreach s [$component services] {'>
<'  if {[catch {$s task}]} {'>
  h->connections-><"[$s name]"> = h->node->serviceClient<
    genom::srv::<"[$s name]">::input, genom::srv::<"[$s name]">::output
    >("/" + std::string(h->instance) + "/<"[$s name]">", true);
  if (!h->connections-><"[$s name]">.exists()) {
    snprintf(ebuffer, sizeof(ebuffer),
             "cannot connect to service %s", "<"[$s name]">");
    h->error = GENOM_CLIENT_ERR_FLAG; goto error;
  }
<'  } else {'>
  h->connections-><"[$s name]"> =
    new gnmix_<"[$s name]">(
      *h->node, "/" + std::string(h->instance) + "/<"[$s name]">");
  if (!h->connections-><"[$s name]">->waitForActionServerToStart(
        ros::Duration(1.))) {
    snprintf(ebuffer, sizeof(ebuffer),
             "cannot connect to action %s", "<"[$s name]">");
    h->error = GENOM_CLIENT_ERR_FLAG; goto error;
  }
<'  }'>
<'}'>

  return h;

error:
  e = h->error;
  if (h->node) delete h->node;
  if (h->instance) free(h->instance);
  if (h->connections) {
<'foreach s [$component services] {'>
<'  if {![catch {$s task}]} {'>
    if (h->connections-><"[$s name]">)
      delete h->connections-><"[$s name]">;
<'  }'>
<'}'>
    delete h->connections;
  }
  free(h);
  ros_fini();
  errno = e;
  return NULL;
}


/* <"[--- ${comp}_fini --------------------------------------------------]"> */

void
genom_<"$comp">_client_fini(genom_client h)
{
  if (h) {
    if (h->node) delete h->node;
    if (h->instance) free(h->instance);
    if (h->connections) {
<'foreach s [$component services] {'>
<'  if {![catch {$s task}]} {'>
      if (h->connections-><"[$s name]">)
        delete h->connections-><"[$s name]">;
<'  }'>
<'}'>
      delete h->connections;
    }
    free(h);
    ros_fini();
  }
}


/* <"[--- ${comp}_name --------------------------------------------------]"> */

const char *
genom_<"$comp">_client_instance(genom_client h)
{
  return h->instance;
}


/* <"[--- ${comp}_eventfd -----------------------------------------------]"> */

int
genom_<"$comp">_client_eventfd(genom_client h)
{
  return ros_eventfd();
}


/* <"[--- ${comp}_service_info ------------------------------------------]"> */

const struct genom_service_info *
genom_<"$comp">_client_service_info(genom_client h, int rqstid)
{
  return ros_service_info(h, rqstid);
}


/* <"[--- ${comp}_done --------------------------------------------------]"> */

int
genom_<"$comp">_client_done(genom_client h, int rqstid)
{
  return ros_done(h, rqstid);
}


/* <"[--- ${comp}_wait --------------------------------------------------]"> */

int
genom_<"$comp">_client_wait(genom_client h, int rqstid)
{
  return ros_waitterm(h, rqstid);
}


/* <"[--- ${comp}_clean -------------------------------------------------]"> */

int
genom_<"$comp">_client_clean(genom_client h, int rqstid)
{
  return ros_clean(h, rqstid);
}


/* <"[--- ${comp}_doevents -----------------------------------------------]"> */

int
genom_<"$comp">_client_doevents(genom_client h)
{
  return ros_doevents(h);
}


/* <"[--- ${comp}_strerror ----------------------------------------------]"> */

const char *
genom_<"$comp">_client_strerror(genom_client h)
{
  extern const char * const genom_<"$comp">_client_emap[];
  extern const unsigned int genom_<"$comp">_client_elen;
  unsigned int e;

  if (!h) e = errno; else e = h->error;
  if ((e & 0xffff0000) == GENOM_SYS_ERR_FLAG) return strerror(e & 0xffff);
  if ((e & 0xffff0000) == GENOM_CLIENT_ERR_FLAG) return ebuffer;

  e = e & 0xffff;
  printf("errmsg %d %s\n", e, genom_<"$comp">_client_emap[e]);
  if (e >= genom_<"$comp">_client_elen - 1)
    e = genom_<"$comp">_client_elen - 1;
  return genom_<"$comp">_client_emap[e];
}
