#
# Copyright (c) 2012 LAAS/CNRS
# All rights reserved.
#
# Redistribution  and  use  in  source  and binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
#   1. Redistributions of  source  code must retain the  above copyright
#      notice and this list of conditions.
#   2. Redistributions in binary form must reproduce the above copyright
#      notice and  this list of  conditions in the  documentation and/or
#      other materials provided with the distribution.
#
# THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
# WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
# MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
# ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
# WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#                                           Anthony Mallet on Thu Aug  2 2012
#

# Ros C client library generation.

template usage {*}{
    "\n"
    "C client library for ros-based components.\n"
    "\n"
    "This template generates a C client library for GenoM component built\n"
    "with the ros middleware.\n"
    "\n"
    "Supported options:\n"
    "  -C, --directory=dir\toutput files in dir\n"
    "  -p, --preserve\tdo not overwrite existing files\n"
    "  -h. --help\t\tprint usage summary (this text)"
}

# defaults
set odir client/c
engine mode +overwrite +move-if-change

# parse options
template options {
    -C - --directory	{ set odir [template arg] }
    -p - --preserve	{ engine mode -overwrite }
    -h - --help		{ puts [template usage]; exit 0 }
}
engine chdir $odir

# add common services to dotgen specification
dotgen parse file [file join [dotgen template builtindir] common/genom.gen]

# parse input
set input [list]
foreach f $argv {
  dotgen parse file $f
  lappend input [file normalize $f]
}

# only one component
set comp [dotgen components]
if {[llength $comp] > 1} {
  template fatal "component compositing not supported"
}

# require utility procs
template require ../../common/typeutil.tcl
template require docutil.tcl
set types [public-types]

# common header for all files (copyright info from .gen file)
set header {/* <"[--- Generated by [dotgen genom version]. Do not edit - ]"> */

<'if {![catch {dotgen input notice} notice]} {
  puts [lang c; comment $notice]
}'>
}

# generate ros .msg/.srv and .action for interface types
template require ../../common/genrosmsg.tcl
set msg_types [genrossrv $comp]
set msg_types [genrosmsg $msg_types]

# client source files
set client_files {
  info.c
  json.c
  init.cc
  client.cc
  msglib.h
  msglib.cc
  portlib.cc
  plugin.c
}
foreach c [dotgen components] {
  template parse args [list $c $types] string $header \
      file client.h file src/[$c name]/c/client.h
  foreach f $client_files {
    template parse args $c string $header \
        file $f file src/[$c name]_[file tail $f]
  }
}

template parse file ../../common/genom-serialize.h file src/genom-serialize.h
template parse args [list $types] \
    file ../../common/serialize.h file src/[$comp name]_serialize.h
template parse file ../../common/portlib.h file src/[$comp name]_portlib.h
template parse args [list [$comp services]] \
    file ../../common/service.h file src/[$comp name]_service.h
template parse args [list [$comp services]] \
    file ../../common/action.h file src/[$comp name]_action.h
template parse args [list $types] \
    file ../../common/typecopy.h file src/[$c name]_typecopy.h

# setup build environment
template parse args [list $msg_types $input] \
    file client.Makefile.am file Makefile.am
template parse file client.configure.ac file configure.ac
template parse file \
    ../../common/autoconf/ax_pthread.m4 file autoconf/ax_pthread.m4
template parse file ../../common/autoconf/ax_boost_base.m4 \
    file autoconf/ax_boost_base.m4
template parse file c-client.pc.in file [$comp name]-c-client.pc.in
template parse file c-client-uninstalled.pc.in \
    file [$comp name]-c-client-uninstalled.pc.in
template link private.h src/private.h

template parse args [list $msg_types] \
    file ../../common/ros.manifest.xml file [$comp name]/manifest.xml

set deps [list]
foreach d [concat [dotgen input deps] [template deps]] {
  lappend deps "regen: $d"
  lappend deps "$d:"
}
engine mode +overwrite -move-if-change
template parse raw [join $deps "\n"]\n file regen
