<'
# Copyright (c) 2011-2012 LAAS/CNRS
# All rights reserved.
#
# Redistribution and use  in source  and binary  forms,  with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   1. Redistributions of  source  code must retain the  above copyright
#      notice and this list of conditions.
#   2. Redistributions in binary form must reproduce the above copyright
#      notice and  this list of  conditions in the  documentation and/or
#      other materials provided with the distribution.
#
#                                      Anthony Mallet on Fri Dec  9 2011
#

if {[llength $argv] != 1} { error "expected arguments: component" }
lassign $argv component

# compute handy shortcuts
set comp [$component name]
set COMP [string toupper [$component name]]
if {[catch { $component version } version]} { set version {}}

lang c
'>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <cstdio>
#include <getopt.h>
#include <libgen.h>
#include <signal.h>
#include <stdlib.h>

#include "ros/ros.h"

#include "<"$comp">_internals.h"

/* options */
static const char usage_string[] = "\
  -b,--background   daemonize\n\
  -i,--name=string  set instance name\n\
     --version      display the version number of the invoked GenoM\n\
  -d,--debug        log extra debugging information on stderr\n\
  -h,--help         print usage summary (this text)\n\
";
static const char shortopts_string[] = "+bi:dh";
static struct option longopts_list[] = {
  { "background",	no_argument,		NULL,	'b'},
  { "name",		required_argument,	NULL,	'i'},
  { "version",		no_argument,		NULL,	-'v'},
  { "debug",		no_argument,		NULL,	'd'},
  { "help",		no_argument,		NULL,	'h'},
  { NULL, 0, NULL, 0}
};

static int daemonize = 0;
static int debug = 0;
const char *genom_instance = "<"$comp">";

static void	apocalypse(int sig);
static void	usage(FILE *channel, char *argv0);


/* --- main ---------------------------------------------------------------- */

/** Initialization and startup
 */
int
main(int argc, char **argv)
{
  int syncfd[2];		/* daemon synchronization pipe */

  int s, c;

  /* parse command line options */
  while (
    (c = getopt_long(argc, argv, shortopts_string, longopts_list, NULL)) != -1)
    switch (c) {
      case 0: break;

      case 'b': daemonize++; break;
      case 'd': debug++; break;
      case 'i': genom_instance = optarg; break;

      case -'v': puts("<"$version">"); exit(0); break;
      case 'h':  usage(stdout, argv[0]);  exit(0); break;

      case '?':
      default:
	usage(stderr, argv[0]);
        exit(1);
        break;
    }
  argc -= optind;
  argv += optind;

  /* daemonize */
  if (daemonize) {
    if (pipe(syncfd) < 0) {
      perror("pipe");
      exit(2);
    }

    switch(fork()) {
      case -1:
        perror("cannot fork");
        exit(2);

      case 0: /* child */ {
        int fd;

        close(syncfd[0]);

        if (setsid() == -1) exit(-1);
        if (chdir("/")) /*noop*/;

        fd = open("/dev/null", O_RDWR, 0);
        if (fd != -1) {
          (void)dup2(fd, STDIN_FILENO);
          if (fd > STDERR_FILENO)
            (void)close(fd);
        }
        break;
      }

      default: /* parent */ {
        unsigned char ch;

        close(syncfd[1]);
        ch = '\0';
        while (read(syncfd[0], &ch, sizeof(ch)) < 0 && errno == EINTR)
          ch = '\0';
        _exit(ch == 'x' ? 0:2);
      }
    }
  }

  signal(SIGINT, apocalypse);
  signal(SIGQUIT, apocalypse);
  signal(SIGTERM, apocalypse);

  ros::init(argc, argv, genom_instance, ros::init_options::NoSigintHandler);
  s = <"$comp">_cntrl_task(genom_instance, daemonize ? syncfd[1] : -1);

  return s;
}


/* --- signal handler ------------------------------------------------------ */

static void
apocalypse(int sig)
{
  ros::requestShutdown();
}


/* --- usage --------------------------------------------------------------- */

/** Print usage on a channel.
 *
 * \param[in] channel	Output channel
 * \param[in] argv0	Program invocation name
 */

static void
usage(FILE *channel, char *argv0)
{
  fprintf(channel,
	  "<"[string trim "$comp $version"]"> GenoM component\n\n"
	  "  %1$s [-i name] [-b] [-d]\n"
	  "  %1$s [-h|--version]\n"
	  "\n%2$s",
	  basename(argv0), usage_string);
}


/* --- log ----------------------------------------------------------------- */

void
genom_log_info(const char *format, ...)
{
  va_list va;

  printf("%s: ", genom_instance);
  va_start(va, format);
  vprintf(format, va);
  va_end(va);
  printf("\n");
}

void
genom_log_warn(const char *format, ...)
{
  va_list va;

  fprintf(stderr, "%s: ", genom_instance);
  va_start(va, format);
  vfprintf(stderr, format, va);
  va_end(va);
  fprintf(stderr, "\n");
}

void
genom_log_debug(const char *format, ...)
{
  va_list va;
  if (!debug) return;

  fprintf(stderr, "%s: ", genom_instance);
  va_start(va, format);
  vfprintf(stderr, format, va);
  va_end(va);
  fprintf(stderr, "\n");
}
